using MediatR;
using cuqmbr.TravelGuide.Application.Common.Interfaces.Persistence;
using AutoMapper;
using cuqmbr.TravelGuide.Application.Common.Models;
using cuqmbr.TravelGuide.Application.Common.Extensions;

namespace cuqmbr.TravelGuide.Application.Aircrafts.Queries.GetAircraftsPage;

public class GetAircraftsPageQueryHandler :
    IRequestHandler<GetAircraftsPageQuery, PaginatedList<AircraftDto>>
{
    private readonly UnitOfWork _unitOfWork;
    private readonly IMapper _mapper;

    public GetAircraftsPageQueryHandler(
        UnitOfWork unitOfWork,
        IMapper mapper)
    {
        _unitOfWork = unitOfWork;
        _mapper = mapper;
    }

    public async Task<PaginatedList<AircraftDto>> Handle(
        GetAircraftsPageQuery request,
        CancellationToken cancellationToken)
    {
        var paginatedList = await _unitOfWork.AircraftRepository.GetPageAsync(
            e =>
                (e.Number.ToLower().Contains(request.Search.ToLower()) ||
                e.Model.ToLower().Contains(request.Search.ToLower())) &&
                (request.CapacityGreaterOrEqualThan != null
                    ? e.Capacity >= request.CapacityGreaterOrEqualThan
                    : true) &&
                (request.CapacityLessOrEqualThan != null
                    ? e.Capacity <= request.CapacityLessOrEqualThan
                    : true),
            request.PageNumber, request.PageSize,
            cancellationToken);

        var mappedItems = _mapper
            .ProjectTo<AircraftDto>(paginatedList.Items.AsQueryable());

        mappedItems = QueryableExtension<AircraftDto>
            .ApplySort(mappedItems, request.Sort);

        _unitOfWork.Dispose();

        return new PaginatedList<AircraftDto>(
            mappedItems.ToList(),
            paginatedList.TotalCount, request.PageNumber,
            request.PageSize);
    }
}
