using MediatR;
using cuqmbr.TravelGuide.Application.Common.Interfaces.Persistence;
using cuqmbr.TravelGuide.Application.Common.Exceptions;
using AutoMapper;

namespace cuqmbr.TravelGuide.Application.VehicleEnrollments
    .Queries.GetVehicleEnrollment;

public class GetVehicleEnrollmentQueryHandler :
    IRequestHandler<GetVehicleEnrollmentQuery, VehicleEnrollmentDto>
{
    private readonly UnitOfWork _unitOfWork;
    private readonly IMapper _mapper;

    public GetVehicleEnrollmentQueryHandler(
        UnitOfWork unitOfWork,
        IMapper mapper)
    {
        _unitOfWork = unitOfWork;
        _mapper = mapper;
    }

    public async Task<VehicleEnrollmentDto> Handle(
        GetVehicleEnrollmentQuery request,
        CancellationToken cancellationToken)
    {
        var entity = await _unitOfWork.VehicleEnrollmentRepository.GetOneAsync(
            e => e.Guid == request.Guid, e => e.RouteAddressDetails,
            cancellationToken);

        if (entity == null)
        {
            throw new NotFoundException();
        }

        // Hydrate vehicle enrollment with address information

        var routeAddresses = await _unitOfWork.RouteAddressRepository
            .GetPageAsync(
                e =>
                    entity.RouteAddressDetails
                    .Select(rad => rad.RouteAddressId)
                    .Contains(e.Id),
                e => e.Address.City.Region.Country,
                1, entity.RouteAddressDetails.Count(),
                cancellationToken);

        foreach (var rad in entity.RouteAddressDetails)
        {
            rad.RouteAddress = routeAddresses.Items
                .First(ra => ra.Id == rad.RouteAddressId);
        }

        _unitOfWork.Dispose();

        return _mapper.Map<VehicleEnrollmentDto>(entity);
    }
}
